import ktc
import sys
import json
import torch
import numpy as np
import trt_pose.coco

from PIL import Image, ImageDraw
from trt_pose.draw_objects import DrawObjects
from trt_pose.parse_objects import ParseObjects


'''
img is PIL format
'''
def draw_keypoints(img, key):
    thickness = 5
    w, h = img.size
    draw = ImageDraw.Draw(img)
    #draw Rankle -> RKnee (16-> 14)
    if all(key[16]) and all(key[14]):
        draw.line([ int(key[16][2] * w), int(key[16][1] * h), int(key[14][2] * w), int(key[14][1] * h)],width = thickness, fill=(51,51,204))
    #draw RKnee -> Rhip (14-> 12)
    if all(key[14]) and all(key[12]):
        draw.line([ int(key[14][2] * w), int(key[14][1] * h), int(key[12][2] * w), int(key[12][1] * h)],width = thickness, fill=(51,51,204))
    #draw Rhip -> Lhip (12-> 11)
    if all(key[12]) and all(key[11]):
        draw.line([ int(key[12][2] * w), int(key[12][1] * h), int(key[11][2] * w), int(key[11][1] * h)],width = thickness, fill=(51,51,204))
    #draw Lhip -> Lknee (11-> 13)
    if all(key[11]) and all(key[13]):
        draw.line([ int(key[11][2] * w), int(key[11][1] * h), int(key[13][2] * w), int(key[13][1] * h)],width = thickness, fill=(51,51,204))
    #draw Lknee -> Lankle (13-> 15)
    if all(key[13]) and all(key[15]):
        draw.line([ int(key[13][2] * w), int(key[13][1] * h), int(key[15][2] * w), int(key[15][1] * h)],width = thickness, fill=(51,51,204))

    #draw Rwrist -> Relbow (10-> 8)
    if all(key[10]) and all(key[8]):
        draw.line([ int(key[10][2] * w), int(key[10][1] * h), int(key[8][2] * w), int(key[8][1] * h)],width = thickness, fill=(255,255,51))
    #draw Relbow -> Rshoulder (8-> 6)
    if all(key[8]) and all(key[6]):
        draw.line([ int(key[8][2] * w), int(key[8][1] * h), int(key[6][2] * w), int(key[6][1] * h)],width = thickness, fill=(255,255,51))
    #draw Rshoulder -> Lshoulder (6-> 5)
    if all(key[6]) and all(key[5]):
        draw.line([ int(key[6][2] * w), int(key[6][1] * h), int(key[5][2] * w), int(key[5][1] * h)],width = thickness, fill=(255,255,0))
    #draw Lshoulder -> Lelbow (5-> 7)
    if all(key[5]) and all(key[7]):
        draw.line([ int(key[5][2] * w), int(key[5][1] * h), int(key[7][2] * w), int(key[7][1] * h)],width = thickness, fill=(51,255,51))
    #draw Lelbow -> Lwrist (7-> 9)
    if all(key[7]) and all(key[9]):
        draw.line([ int(key[7][2] * w), int(key[7][1] * h), int(key[9][2] * w), int(key[9][1] * h)],width = thickness, fill=(51,255,51))

    #draw Rshoulder -> RHip (6-> 12)
    if all(key[6]) and all(key[12]):
        draw.line([ int(key[6][2] * w), int(key[6][1] * h), int(key[12][2] * w), int(key[12][1] * h)],width = thickness, fill=(153,0,51))
    #draw Lshoulder -> LHip (5-> 11)
    if all(key[5]) and all(key[11]):
        draw.line([ int(key[5][2] * w), int(key[5][1] * h), int(key[11][2] * w), int(key[11][1] * h)],width = thickness, fill=(153,0,51))


    #draw nose -> Reye (0-> 2)
    if all(key[0][1:]) and all(key[2]):
        draw.line([ int(key[0][2] * w), int(key[0][1] * h), int(key[2][2] * w), int(key[2][1] * h)],width = thickness, fill=(219,0,219))

    #draw Reye -> Rear (2-> 4)
    if all(key[2]) and all(key[4]):
        draw.line([ int(key[2][2] * w), int(key[2][1] * h), int(key[4][2] * w), int(key[4][1] * h)],width = thickness, fill=(219,0,219))

    #draw nose -> Leye (0-> 1)
    if all(key[0][1:]) and all(key[1]):
        draw.line([ int(key[0][2] * w), int(key[0][1] * h), int(key[1][2] * w), int(key[1][1] * h)],width = thickness, fill=(219,0,219))

    #draw Leye -> Lear (1-> 3)
    if all(key[1]) and all(key[3]):
        draw.line([ int(key[1][2] * w), int(key[1][1] * h), int(key[3][2] * w), int(key[3][1] * h)],width = thickness, fill=(219,0,219))

    #draw nose -> neck (0-> 17)
    if all(key[0][1:]) and all(key[17]):
        draw.line([ int(key[0][2] * w), int(key[0][1] * h), int(key[17][2] * w), int(key[17][1] * h)],width = thickness, fill=(255,255,0))
    return img


'''
hnum: 0 based human index
kpoint : index + keypoints (float type range : 0.0 ~ 1.0 ==> later multiply by image width, height)
'''
def get_keypoint(humans, hnum, peaks):
    #check invalid human index
    kpoint = []
    human = humans[0][hnum]
    C = human.shape[0]
    for j in range(C):
        k = int(human[j])
        if k >= 0:
            peak = peaks[0][j][k]   # peak[1]:width, peak[0]:height
            peak = (j, float(peak[0]), float(peak[1]))
            kpoint.append(peak)
            print('index:%d : success [%5.3f, %5.3f]'%(j, peak[1], peak[2]) )
        else:    
            peak = (j, None, None)
            kpoint.append(peak)
            print('index:%d : None'%(j) )
    return kpoint





def normalization(x, mode, **kwargs):
    """Normalize a Numpy array encoding a batch of images.
    # Arguments
        x: Input array, 3D or 4D.
        mode: One of "kneron","caffe", "tf" or "torch".
            - caffe: will convert the images from RGB to BGR,
                then will zero-center each color channel with
                respect to the ImageNet dataset,
                without scaling.
            - tf: will scale pixels between -1 and 1,
                sample-wise.
            - torch: will scale pixels between 0 and 1 and then
                will normalize each channel with respect to the
                ImageNet dataset.
    # Returns
        Preprocessed Numpy array.
    """

    x = x.astype(float)

    if mode == 'none':
        return x

    if mode == 'yolo': # 0 -1  8-7
        x /= 255.
        return x

    if mode == 'kneron': #-0.5～0.5
        x /= 256.
        x -= 0.5
        return x

    if mode == 'tensorflow': # -1 ～ 1 8-7
        x /= 127.5
        x -= 1.
        return x

    if mode == 'pytorch': #-2.5～2.5 8-6
        x /= 255.
        mean = [0.485, 0.456, 0.406]
        std = [0.229, 0.224, 0.225]
        x[..., 0] -= mean[0]
        x[..., 1] -= mean[1]
        x[..., 2] -= mean[2]
        if std is not None:
            x[..., 0] /= std[0]
            x[..., 1] /= std[1]
            x[..., 2] /= std[2]
        return x


def preprocess(input_image):
    image = Image.open(input_image)
    image_data = image.convert("RGB")
    image_data = Image.fromarray(np.array(image_data)[...,::-1])
    image_data = np.array(image_data.resize((224, 224), Image.BILINEAR))
    print(image_data.shape)

    image_data = normalization(image_data, "pytorch")
    image_data = np.expand_dims(image_data, 0)  # Add batch dimension.

    post_dict = {
        "image_shape": [image.size[1], image.size[0]]
    }
    return [image_data], post_dict


# def postprocess(inf_results, pre_data):
#     new_data = [torch.from_numpy(data) for data in inf_results]
#     counts, objects, peaks = parse_objects(new_data[1], new_data[2])
#     image_shape = pre_data["image_shape"]
#     # for i in range(counts[0]):
#     #     print("Human index:%d "%( i ))
#     #     kpoint = get_keypoint(objects, i, peaks)
#     #     org = draw_keypoints(org, kpoint)
#     print("Human count:%d len:%d "%(counts[0], len(counts)))
#     return [counts[0], len(counts)]


def run_example(input_image):
    preprocess_data, post_dict = preprocess(input_image)

    # ONNX model
    # inf_results = ktc.kneron_inference(preprocess_data, onnx_file="output.onnx", input_names=["input"])

    # print(inf_results)
    # print("=======================================1")
    # results = postprocess(inf_results, post_dict)
    # print(results)

    # BIE model
    # inf_results = ktc.kneron_inference(preprocess_data, bie_file="/data1/fpAnalyser/output.quan.wqbi.bie", input_names=["input"])

    # print(inf_results)
    # print("=======================================2")
    # results = postprocess(inf_results, post_dict)
    # print(results)

    # NEF model
    inf_results = ktc.kneron_inference(preprocess_data, "/data1/batch_compile/models_720.nef", radix=7)

    print(inf_results)
    print("=======================================3")
    # results = postprocess(inf_results, post_dict)
    # print(results)


# with open('/data1/human_pose.json', 'r') as f:
#     human_pose = json.load(f)

# topology = trt_pose.coco.coco_category_to_topology(human_pose)
# parse_objects = ParseObjects(topology)
# draw_objects = DrawObjects(topology)

input_image = sys.argv[1]
run_example(input_image)